import { ConsoleMixin } from './ConsoleMixin'
import { DatetimeMixin } from './DatetimeMixin'
import { HttpMixin } from './HttpMixin'
import { NotificationsMixin } from './NotificationsMixin'

export const OBJECT_TYPE = Object.freeze({
    CONTACT: 'contact',
    DOMAIN: 'domain',
    NSSET: 'nsset',
    KEYSET: 'keyset',
    REGISTRAR: 'registrar',
})

export const DOMAIN_QUERY = Object.freeze({
    START: 'start',
    END: 'end',
})

export const CONTACT_MAIN_ITEMS = Object.freeze({
    ORGANIZATION: 'organization',
    NAME: 'name',
    ADDITIONAL_IDENTIFIER: 'additional_identifier',
    VAT_IDENTIFICATION_NUMBER: 'vat_identification_number',
    TELEPHONE: 'telephone',
    FAX: 'fax',
    EMAIL: 'email',
    NOTIFY_EMAIL: 'notify_email',
    WARNING_LETTER: 'warning_letter',
})

export const CONTACT_ADDRESS_ITEMS = Object.freeze([
    'place',
    'mailing_address',
    'billing_address',
    'shipping_address1',
    'shipping_address2',
    'shipping_address3',
])

export const REGISTRAR_MAIN_ITEMS = Object.freeze([
    'organization',
    'name',
    'telephone',
    'fax',
    'emails',
    'url',
    'is_system_registrar',
    'company_registration_number',
    'vat_identification_number',
    'is_vat_payer',
    'variable_symbol',
    'payment_memo_regex',
])

export const EVENT_MAIN_ITEMS = Object.freeze([
    'registered',
    'transferred',
    'updated',
    'unregistered',
])

export const CONTACT_ROLES = Object.freeze({
    HOLDER: 'holder',
    ADMIN: 'admin',
    KEYSET_TECH_CONTACT: 'keyset_tech_contact',
    NSSET_TECH_CONTACT: 'nsset_tech_contact',
})

export const RELATED_DOMAIN_TABLE_COLUMNS = Object.freeze({
    FQDN: 'fqdn',
    SPONSORING_REGISTRAR: 'sponsoring_registrar',
    ROLES: 'roles',
    NSSET: 'nsset',
    KEYSET: 'keyset',
    EXPIRES_AT: 'expires_at',
})

export const RegistryMixin = {
    mixins: [
        NotificationsMixin,
        DatetimeMixin,
        ConsoleMixin,
        HttpMixin,
    ],

    created() {
        this.OBJECT_TYPE = OBJECT_TYPE
        this.CONTACT_MAIN_ITEMS = CONTACT_MAIN_ITEMS
        this.CONTACT_ADDRESS_ITEMS = CONTACT_ADDRESS_ITEMS
        this.REGISTRAR_MAIN_ITEMS = REGISTRAR_MAIN_ITEMS
        this.EVENT_MAIN_ITEMS = EVENT_MAIN_ITEMS
    },

    methods: {
        get_object_detail_url(object_type, object_id) {
            return `/registry/${object_type}/${object_id}/`
        },

        get_object_history_url(object_type, object_id) {
            return `/registry/${object_type}_history/${object_id}/`
        },

        get_registrar_detail_url(handle) {
            const encoded_handle = encodeURIComponent(handle)
            return `/registry/registrar/${encoded_handle}/`
        },

        get_registrar_list_url() {
            return '/registry/ajax/registrar_list/'
        },

        get_registrar_credit_list_url() {
            return '/registry/ajax/registrar_credit_list/'
        },

        get_search_ajax_url(query_values, object_type, search_items) {
            const params = this.get_search_params(query_values, object_type, search_items)
            return `/registry/ajax/search_${object_type}/?${params.toString()}`
        },

        get_history_search_ajax_url(query_values, search_date, object_type, search_items) {
            const params = this.get_search_params(
                query_values,
                object_type,
                search_items,
                null,
                this.history_search_period(search_date),
            )
            return `/registry/ajax/search_${object_type}_history/?${params.toString()}`
        },

        get_auth_info_ajax_url(object_type, object_id, history_datetime = null) {
            if (!history_datetime)
                return `/registry/ajax/auth_info/${object_type}/${object_id}/`
            else
                return `/registry/ajax/auth_info/${object_type}/${object_id}/history/${history_datetime}/`
        },

        get_object_info_ajax_url(object_type, object_id, history_spec = null) {
            if (history_spec === null)
                return `/registry/ajax/${object_type}_info/${object_id}/`
            else
                return `/registry/ajax/${object_type}_info/${object_id}/history/${history_spec}/`
        },

        get_object_state_ajax_url(object_type, object_id) {
            return `/registry/ajax/${object_type}_state/${object_id}/`
        },

        get_object_state_history_ajax_url(object_type, object_id) {
            return `/registry/ajax/${object_type}_state_history/${object_id}/`
        },

        get_contact_representative_url(contact_id) {
            return `/registry/ajax/contact_representative/${contact_id}/`
        },

        get_contact_representative_change_url(contact_id) {
            return `/registry/ajax/contact_representative_change/${contact_id}/`
        },

        get_contact_representative_delete_url(contact_id) {
            return `/registry/ajax/contact_representative_delete/${contact_id}/`
        },

        get_search_params(
            query_values,
            object_type,
            search_items,
            search_date = null,
            valid_from = null,
            reg_query = null,
            reg_zone = null,
            active_tab = null,
        ) {
            const params = new URLSearchParams()
            for (const query_item of query_values) {
                params.append('query', query_item)
            }
            if (search_date && search_date !== this.SEARCH_HISTORY_PERIOD.NOW)
                params.append('history', search_date)
            if (valid_from)
                params.append('valid_from', valid_from)
            params.append('type', object_type)
            for (const item of search_items) {
                params.append('items', item)
            }

            if (reg_query !== null && reg_query !== '') {
                params.append('reg_query', reg_query)
            }
            if (reg_zone !== null) params.append('reg_zone', reg_zone)
            if (active_tab !== null) params.append('tab', active_tab)

            return params
        },

        get_search_url() {
            return '/registry/search/'
        },

        get_login_url() {
            return `/accounts/login/${location.search}`
        },

        get_logout_url() {
            return `/accounts/logout/`
        },

        get_admin_url() {
            return `/admin/`
        },

        // https://www.iana.org/assignments/dns-sec-alg-numbers/dns-sec-alg-numbers.xhtml
        get_dnskey_alg_label(alg_id) {
            if (alg_id < 0 || alg_id > 255)
                return `${alg_id} ${this.$t('keyset:is_out_of_range')}`

            if (alg_id >= 17 && alg_id <= 122)
                return this.$t('keyset:unassigned')

            return this.$t([`keyset:dns_key_alg.alg${alg_id}`, 'keyset:reserved'])
        },

        // https://www.iana.org/assignments/dnskey-flags/dnskey-flags.xhtml#dnskey-flags-1
        //  0-6  Unassigned
        //    7  ZONE
        //    8  REVOKE
        // 9-14  Unassigned
        //   15  Secure Entry Point (SEP)
        get_dnskey_flag_labels(flags) {
            if (flags < 0 || flags > 0xffff)
                return `${flags} ${this.$t('keyset:is_out_of_range')}`

            const labels = []
            if (flags & 0b0000000000000001)
                labels.push(this.$t('keyset:dns_key_flag.sep'))
            if (flags & 0b0000000100000000)
                labels.push(this.$t('keyset:dns_key_flag.zone'))
            if (flags & 0b0000000010000000)
                labels.push(this.$t('keyset:dns_key_flag.revoke'))

            return labels.join(', ')
        },

        // https://www.iana.org/assignments/dns-key-rr/dns-key-rr.xhtml#dns-key-rr-1
        // 1-2, 4, 255 Reserved
        // 5-254       Unassigned
        // 3           DNSSEC
        get_dnskey_protocol_label(protocol_id) {
            if (protocol_id < 0 || protocol_id > 255)
                return `${protocol_id} ${this.$t('keyset:is_out_of_range')}`

            if (protocol_id === 3)
                return this.$t('keyset:dnssec')
            else if (protocol_id >= 5 && protocol_id <= 254)
                return this.$t('keyset:unassigned')
            else
                return this.$t('keyset:reserved')
        },

        async get_object_info(object_type, uuid, history_spec = null) {
            let response
            try {
                response = await this.http_get(this.get_object_info_ajax_url(object_type, uuid, history_spec))
            } catch (error) {
                this.notify_message(this.ERROR_BACKEND_NOT_AVAILABLE)
                this.log_error(error)
                return null
            }
            return response.data
        },

        async get_nsset_info(uuid, history_spec = null) {
            return this.get_object_info(OBJECT_TYPE.NSSET, uuid, history_spec)
        },

        async get_keyset_info(uuid, history_spec = null) {
            return this.get_object_info(OBJECT_TYPE.KEYSET, uuid, history_spec)
        },

        async get_countries() {
            let response
            try {
                response = await this.http_get('/countries/')
            } catch (error) {
                this.notify_message(this.ERROR_SOMETHING_WENT_WRONG)
                this.log_error(error)
                return null
            }
            return response.data
        },

        fqdn_sort(arr) {
            return arr.map(fqdn => fqdn.split('.').reverse()).sort().map(fqdn => fqdn.reverse().join('.'))
        },
        /**
         * Get state flags data
         *
         * @param { String } object_type
         * @param { String } object_id
         * @returns { Object } response
         * @returns { Object } null - In case of server error
         */
        async get_state_flags_data(object_type, object_id) {
            let response
            try {
                response = await this.http_get(this.get_object_state_ajax_url(object_type, object_id))
            } catch (error) {
                this.notify_message(this.ERROR_BACKEND_NOT_AVAILABLE)
                this.log_error(error)
                return null
            }
            return response.data
        },
    },
}
