from decimal import Decimal
from unittest.mock import patch

from django.test import TestCase, override_settings
from django.urls import reverse

from ferda.backend.exceptions import RegistrarDoesNotExist
from ferda.constants import LogEntryType, LoggerResult
from ferda.models import User
from ferda.tests.utils import LoggerMixin


@override_settings(ROOT_URLCONF='ferda.tests.urls')
class TestRegistrarListView(LoggerMixin, TestCase):
    """Registrar list view test."""

    def setUp(self):
        super().setUp()
        self.user = User.objects.create_superuser('admin', 'admin@example.com', 'password')
        self.client.force_login(self.user)

    @patch('ferda.backend.BACKEND.get_registrar_handles', autospec=True)
    @patch('ferda.backend.BACKEND.get_registrar_info_future', autospec=True)
    @patch('ferda.backend.BACKEND.get_registrar_zone_access_future', autospec=True)
    def test_get(self, zone_mock, info_mock, handles_mock):
        handles_mock.return_value = ['REG-SW']
        info_mock.return_value.result.return_value = {
            'registrar_handle': 'REG-SW',
            'name': 'Star Wars Inc.',
            'organization': 'Star Wars Inc.'
        }
        zone_mock.return_value.result.return_value = {'cz': True, '0.2.4.e164.arpa': False}

        response = self.client.get(reverse('registry:registrar:ajax_registrar_list'))
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(
            response.content, {
                'registrar_list': [{
                    'handle': 'REG-SW',
                    'name': 'Star Wars Inc.',
                    'organization': 'Star Wars Inc.',
                    'zone_access': {'0.2.4.e164.arpa': False, 'cz': True},
                }]
            }
        )
        self.assertLog(
            LogEntryType.REGISTRAR_LIST,
            LoggerResult.SUCCESS,
            input_properties=None,
            references={'registrar': ['REG-SW']},
        )

    @patch('ferda.backend.BACKEND.get_registrar_handles', autospec=True)
    @patch('ferda.backend.BACKEND.get_registrar_info_future', autospec=True)
    @patch('ferda.backend.BACKEND.get_registrar_zone_access_future', autospec=True)
    def test_get_registrar_not_found(self, zone_mock, info_mock, handles_mock):
        handles_mock.return_value = ['REG-SW']
        info_mock.return_value.result.side_effect = RegistrarDoesNotExist
        zone_mock.return_value.result.side_effect = RegistrarDoesNotExist

        response = self.client.get(reverse('registry:registrar:ajax_registrar_list'))
        self.assertEqual(response.status_code, 404)
        self.assertLog(
            LogEntryType.REGISTRAR_LIST,
            LoggerResult.FAIL,
            input_properties=None,
            references=None,
        )


@override_settings(ROOT_URLCONF='ferda.tests.urls')
class TestRegistrarCreditListView(LoggerMixin, TestCase):
    """Registrar credit list view test."""

    def setUp(self):
        super().setUp()
        self.user = User.objects.create_superuser('admin', 'admin@example.com', 'password')
        self.client.force_login(self.user)

    @patch('ferda.backend.BACKEND.get_registrar_handles', autospec=True)
    @patch('ferda.backend.BACKEND.get_registrar_credit_future', autospec=True)
    def test_get(self, credit_mock, handles_mock):
        handles_mock.return_value = ['REG-SW']
        credit_mock.return_value.result.return_value = {'cz': Decimal(42), '0.2.4.e164.arpa': Decimal(0)}

        response = self.client.get(
            reverse('registry:registrar:ajax_registrar_credit_list')
        )
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(
            response.content, {
                'registrar_credits': {
                    'REG-SW': {'cz': '42', '0.2.4.e164.arpa': '0'},
                }
            }
        )

        self.assertLog(
            LogEntryType.REGISTRAR_CREDIT_LIST,
            LoggerResult.SUCCESS,
            input_properties=None,
            references={'registrar': ['REG-SW']},
        )

    @patch('ferda.backend.BACKEND.get_registrar_handles', autospec=True)
    @patch('ferda.backend.BACKEND.get_registrar_credit_future', autospec=True)
    def test_get_registrar_not_found(self, credit_mock, handles_mock):
        handles_mock.return_value = ['REG-SW']
        credit_mock.return_value.result.side_effect = RegistrarDoesNotExist

        response = self.client.get(reverse('registry:registrar:ajax_registrar_credit_list'))
        self.assertEqual(response.status_code, 404)
        self.assertLog(
            LogEntryType.REGISTRAR_CREDIT_LIST,
            LoggerResult.FAIL,
            input_properties=None,
            references=None,
        )
