from decimal import Decimal
from unittest.mock import call, patch

from django.contrib.auth.models import Permission
from django.test import TestCase, override_settings
from django.urls import reverse
from grill import LogProperty

from ferda.backend.exceptions import RegistrarDoesNotExist
from ferda.constants import LogEntryType, LoggerResult
from ferda.models import User
from ferda.tests.utils import LoggerMixin


@override_settings(ROOT_URLCONF='ferda.tests.urls')
class TestRegistrarDetailView(LoggerMixin, TestCase):

    def setUp(self):
        super().setUp()

        self.user = User.objects.create_superuser('admin', 'admin@example.com', 'password')
        self.client.force_login(self.user)

    @patch('ferda.backend.BACKEND.get_registrar_zone_access')
    @patch('ferda.backend.BACKEND.get_registrar_credit')
    @patch('ferda.backend.BACKEND.get_registrar_info')
    def test_get(self, info_mock, credit_mock, zone_mock):
        info_mock.return_value = {'registrar_handle': 'REG-SW', 'name': 'Star Wars Inc.'}
        credit_mock.return_value = {'cz': Decimal('42000.00'), '0.2.4.e164.arpa': Decimal('0.0')}
        zone_mock.return_value = {'cz': True, '0.2.4.e164.arpa': False}
        response = self.client.get(reverse('registry:registrar:detail', kwargs={'handle': 'REG-SW'}))

        self.assertEqual(info_mock.mock_calls, [call('REG-SW')])
        self.assertEqual(response.context['registrar_data'], {
            'registrar_handle': 'REG-SW',
            'name': 'Star Wars Inc.',
            'credit': {
                'cz': Decimal('42000.00'),
                '0.2.4.e164.arpa': Decimal('0.0'),
            },
            'zone_access': {
                'cz': True,
                '0.2.4.e164.arpa': False,
            },
        })
        self.assertContains(response, '<title>Registry | Ferda</title>', html=True)
        self.assertLog(
            LogEntryType.REGISTRAR_DETAIL,
            LoggerResult.SUCCESS,
            input_properties={'registrar_handle': LogProperty('REG-SW', None)},
            references={'registrar': 'REG-SW'},
        )

    @patch('ferda.backend.BACKEND.get_registrar_zone_access')
    @patch('ferda.backend.BACKEND.get_registrar_credit')
    @patch('ferda.backend.BACKEND.get_registrar_info')
    def test_get_without_credit(self, info_mock, credit_mock, zone_mock):
        unauthorized_user = User.objects.create_user(
            'user', 'user@example.com', 'password', first_name='Luke', last_name='Skywalker')
        unauthorized_user.user_permissions.set(
            Permission.objects.filter(codename__in=('can_view_registry',)))
        self.client.force_login(unauthorized_user)

        info_mock.return_value = {'registrar_handle': 'REG-SW', 'name': 'Star Wars Inc.'}
        credit_mock.return_value = {'cz': Decimal('42000.00'), '0.2.4.e164.arpa': Decimal('0.0')}
        zone_mock.return_value = {'cz': True, '0.2.4.e164.arpa': False}
        response = self.client.get(reverse('registry:registrar:detail', kwargs={'handle': 'REG-SW'}))

        self.assertEqual(response.context['registrar_data']['credit'], {
            'cz': None,
            '0.2.4.e164.arpa': None,
        })
        self.assertContains(response, '<title>Registry | Ferda</title>', html=True)

    @patch('ferda.backend.BACKEND.get_registrar_info')
    def test_registrar_not_found(self, info_mock):
        info_mock.side_effect = RegistrarDoesNotExist
        response = self.client.get(
            reverse('registry:registrar:detail', kwargs={'handle': 'REG-SW'})
        )
        self.assertEqual(info_mock.mock_calls, [call('REG-SW')])
        self.assertEqual(response.status_code, 404)
        self.assertLog(
            LogEntryType.REGISTRAR_DETAIL,
            LoggerResult.FAIL,
            input_properties={'registrar_handle': LogProperty('REG-SW', None)},
        )
