"""Registrar related ajax views."""
from typing import Any, Dict, List, Tuple

from django.contrib.auth.decorators import login_required, permission_required
from django.http import Http404, JsonResponse
from django.utils.decorators import method_decorator
from django.utils.translation import gettext as _
from django.views.decorators.cache import cache_page
from django.views.generic.base import View

from ferda.backend import BACKEND, LOGGER
from ferda.backend.exceptions import RegistrarDoesNotExist
from ferda.constants import LOGGER_SESSION_KEY, LogEntryType, LoggerResult


@method_decorator(login_required, name='dispatch')
@method_decorator(permission_required('ferda.can_view_registry'), name='dispatch')
@method_decorator(cache_page(60 * 5), name='dispatch')
class RegistrarListView(View):
    """Registar list view."""

    def get(self, request):
        """Fetch list of registrars from backend."""
        log_entry = LOGGER.create(
            LogEntryType.REGISTRAR_LIST,
            source_ip=self.request.META.get('REMOTE_ADDR', ''),
            session_id=self.request.session[LOGGER_SESSION_KEY],
        )
        registrar_list: List[Dict[str, Any]] = []

        with log_entry:
            registrar_handles = BACKEND.get_registrar_handles()
            tasks: List[Tuple[str, Any, Any]] = []

            for handle in registrar_handles:
                tasks.append((
                    handle,
                    BACKEND.get_registrar_info_future(handle),
                    BACKEND.get_registrar_zone_access_future(handle),
                ))

            try:
                for handle, info_future, zone_access_future in tasks:
                    info = info_future.result()
                    zone_access = zone_access_future.result()

                    registrar_list.append({
                        'handle': handle,
                        'name': info['name'],
                        'organization': info['organization'],
                        'zone_access': zone_access,
                    })
            except RegistrarDoesNotExist:
                log_entry.result = LoggerResult.FAIL
                raise Http404(_('Registrar does not exist'))

            log_entry.result = LoggerResult.SUCCESS
            log_entry.references['registrar'] = registrar_handles

        return JsonResponse({'registrar_list': registrar_list})


@method_decorator(login_required, name='dispatch')
@method_decorator(permission_required(['ferda.can_view_registry', 'ferda.can_view_registrar_credit']), name='dispatch')
@method_decorator(cache_page(60 * 5), name='dispatch')
class RegistrarCreditListView(View):
    """
    Registar credit list view.

    This view is separated from RegistrarListView for easier caching.
    """

    def get(self, request):
        """Fetch list of registrar credit from backend."""
        log_entry = LOGGER.create(
            LogEntryType.REGISTRAR_CREDIT_LIST,
            source_ip=self.request.META.get('REMOTE_ADDR', ''),
            session_id=self.request.session[LOGGER_SESSION_KEY],
        )
        registrar_credits: Dict[str, Any] = {}
        tasks: List[Tuple[str, Any]] = []

        with log_entry:
            registrar_handles = BACKEND.get_registrar_handles()
            for handle in registrar_handles:
                tasks.append((handle, BACKEND.get_registrar_credit_future(handle)))
            try:
                for handle, credit_future in tasks:
                    registrar_credits[handle] = credit_future.result()
            except RegistrarDoesNotExist:
                log_entry.result = LoggerResult.FAIL
                raise Http404(_('Registrar does not exist'))
            log_entry.result = LoggerResult.SUCCESS
            log_entry.references['registrar'] = registrar_handles

        return JsonResponse({'registrar_credits': registrar_credits})
