"""Registrar views."""

from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.http import Http404
from django.utils.translation import gettext as _
from django.views.generic import TemplateView
from grill import LogProperty

from ferda.backend import BACKEND, LOGGER
from ferda.backend.exceptions import RegistrarDoesNotExist
from ferda.constants import LOGGER_SESSION_KEY, LogEntryType, LoggerResult

from .mixins import RegistryPluginMixin


class RegistrarDetailView(LoginRequiredMixin, PermissionRequiredMixin, RegistryPluginMixin, TemplateView):
    """Registrar detail view."""

    permission_required = 'ferda.can_view_registry'
    template_name = 'ferda/registry/registrar_detail.html'

    def get_context_data(self, handle: str):  # type: ignore
        """Add registrar data to context."""
        log_entry = LOGGER.create(
            LogEntryType.REGISTRAR_DETAIL,
            source_ip=self.request.META.get('REMOTE_ADDR', ''),
            session_id=self.request.session[LOGGER_SESSION_KEY],
            properties={'registrar_handle': LogProperty(handle, None)},
        )
        with log_entry:
            try:
                registrar_info = BACKEND.get_registrar_info(handle)
                registrar_info['credit'] = BACKEND.get_registrar_credit(handle)
                registrar_info['zone_access'] = BACKEND.get_registrar_zone_access(handle)
            except RegistrarDoesNotExist:
                log_entry.result = LoggerResult.FAIL
                raise Http404(_('Registrar does not exist'))
            log_entry.references['registrar'] = handle
            log_entry.result = LoggerResult.SUCCESS

        if not self.request.user.has_perm('ferda.can_view_registrar_credit'):
            registrar_info['credit'] = {zone: None for zone, credit in registrar_info['credit'].items()}

        context = super().get_context_data(handle=handle)
        context['registrar_data'] = registrar_info
        return context
